package com.kringlamobil;

import com.kringlamobil.model.ImageHandler;
import com.kringlamobil.model.ListItem;
import com.kringlamobil.model.SearchServie;

import java.util.List;

import android.app.Activity;
import android.app.ProgressDialog;
import android.graphics.Bitmap;
import android.graphics.drawable.BitmapDrawable;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;
import android.view.animation.AnimationUtils;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

public class ImageScreen extends Activity implements OnTouchListener
{
	private ProgressDialog mProgressDialog;
	private ProgressThread progressThread;
    private float downXValue;
    private int THRESHOLD = 30;
    private List<String> mImages;
    private int mCurser = 0;    
    private Bitmap mImage;
	        
    /** 
     * Handles the screens touch event.
     * If the user slides its finger from right to left and the item
     * has multiple images the next image is loaded and displayed to
     * the user. The opposite gesture left to right, loads the previous
     * image in the list. 
     */
    public boolean onTouch(View v, MotionEvent motionEvent) 
    {        
        switch (motionEvent.getAction())
        {
            case MotionEvent.ACTION_DOWN:
            {
                // Store the X value where the user's finger first is pressed down
                downXValue = motionEvent.getX();
                break;
            }

            case MotionEvent.ACTION_UP:
            {
                // Retrieves the X value where the user released its finger
                float currentX = motionEvent.getX();            
                
                //The User has moved its finger from left to right
                if (downXValue < currentX -THRESHOLD)
                {
                	// If not at position 0: loads the previous image in list
                	if(mCurser != 0)
                	{
                		mCurser -= 1;
        	            mProgressDialog = new ProgressDialog(ImageScreen.this);
        	            mProgressDialog.setMessage("Laddar...");
        	            mProgressDialog.show();
        	            progressThread = new ProgressThread(handler);
        				progressThread.start();
                	}
                }

                //The User has moved its finger from right to left
                if (downXValue > currentX +THRESHOLD)
                {
                	// If not at the lists end: loads the next image in list
                	if((mImages.size()-1)>mCurser)
                	{
                		mCurser += 1;
        	            mProgressDialog = new ProgressDialog(ImageScreen.this);
        	            mProgressDialog.setMessage("Laddar...");
        	            mProgressDialog.show();
        	            progressThread = new ProgressThread(handler);
        				progressThread.start();
                	}
                }
                break;
            }
        }

        return true;
    }
    
    /** 
     * Called when the instance state is being saved. 
     * Saves the current counter value. 
     */
    @Override 
    public void onSaveInstanceState(Bundle outState) 
    {
        outState.putInt("CURSER", mCurser);
        super.onSaveInstanceState(outState); 
    }
    
    /** 
     * Called when the instance state is being restored. 
     * Restores and sets the counter value. 
     */
    @Override
    public void onRestoreInstanceState(Bundle savedInstanceState) 
    {
        super.onRestoreInstanceState(savedInstanceState);
        mCurser = savedInstanceState.getInt("CURSER");
        TextView counter = (TextView) findViewById(R.id.image_counter);
        counter.setText((mCurser+1)+"/" +mImages.size());
    }
    
    /** 
     * Called when the activity is being destroyed. 
     * Saves the current image from being lost when the phone 
     * changes orientation. 
     */
    @Override  
    public Object onRetainNonConfigurationInstance() 
    {   
    	ImageView image = (ImageView) findViewById(R.id.image);
        return ((BitmapDrawable)image.getDrawable()).getBitmap();   
    }
    
    /** Called when the activity starts. */
    @Override
    protected void onStart()
    {
    	super.onStart();
        ImageView image = (ImageView) findViewById(R.id.image);
        Bitmap img = (Bitmap) getLastNonConfigurationInstance();
        if (img == null)
        {
            mProgressDialog = new ProgressDialog(ImageScreen.this);
            mProgressDialog.setMessage("Laddar...");
            mProgressDialog.show();
            progressThread = new ProgressThread(handler);
    		progressThread.start();
        }
        image.setImageBitmap(img);
        
        TextView counter = (TextView) findViewById(R.id.image_counter);
        counter.setText((mCurser+1)+"/" +mImages.size());
    }
	
	/** Called when the activity is first created. */
    public void onCreate(Bundle savedInstanceState) 
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.image);
        RelativeLayout layMain = (RelativeLayout) findViewById(R.id.main_layout);
        layMain.setOnTouchListener((OnTouchListener) this); 
        
		SearchServie service = SearchServie.getInstance();
		List<ListItem> items = service.getItems();
		ListItem item = items.get(getIntent().getIntExtra("Id", 0));
		
        mImages = item.getImages();
    }
    
    /** 
     * Define the Handler that receives messages from the nested thread and 
     * update the GUI accordingly. In this case updating the current image with a newly downloaded.
     */
    final Handler handler = new Handler() 
    {        
        public void handleMessage(Message msg) 
        {
        	if (msg.what == 0)
        	{
        		ImageView image =(ImageView) findViewById(R.id.image);
        		image.setImageBitmap(mImage);
            	image.startAnimation(AnimationUtils.loadAnimation(ImageScreen.this, R.anim.fade));
            	
                TextView counter = (TextView) findViewById(R.id.image_counter);
                counter.setText((mCurser+1)+"/" +mImages.size());
			}
        	mProgressDialog.dismiss();
        }
    };
    
    /** A nested thread that downloads the next image to be display. */
    private class ProgressThread extends Thread 
    {
        Handler mHandler;
       
        ProgressThread(Handler h) 
        {
            mHandler = h;
        }
       
        public void run()
        {
            ImageHandler imageHandler = new ImageHandler();
            mImage = imageHandler.DownloadImage(mImages.get(mCurser));	
        	mHandler.sendEmptyMessage(0);
        }

    }
}
