package com.kringlamobil;

import com.google.android.apps.analytics.GoogleAnalyticsTracker;
import com.kringlamobil.model.SearchServie;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.ImageView;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

public class MainSearchScreen extends Activity {	
	
	GoogleAnalyticsTracker tracker;
	
	private ProgressDialog mProgressDialog;
	private ProgressThread mProgressThread;
	private String mQuery;
	private boolean mOnlyImages;
	private boolean mNoLibris;
	private String mItemType;
	private String mMilieu;
	
	/**
	 * Handles the on click event for the search button. 
	 * If the search textview has data then the search process is continued. 
	 * If not, nothing happens.
	 */
	private OnClickListener onClickListener = new OnClickListener() {
		public void onClick(View v) 
		{
			if( v == findViewById(R.id.submitButton))
			{
			TextView searchText = (TextView) findViewById(R.id.search_text);
			Spinner typeSpinner = (Spinner) findViewById(R.id.type_spinner);
			Spinner milieuSpinner = (Spinner) findViewById(R.id.milieu_spinner);
			CheckBox imageCheckBox = (CheckBox) findViewById(R.id.imageCheckBox);
			CheckBox librisCheckBox = (CheckBox) findViewById(R.id.librisCheckBox);
			
				//Check to see if there is a search string
				if(!"".equals(searchText.getText().toString()))
				{
					mQuery = searchText.getText().toString();
					mOnlyImages = imageCheckBox.isChecked();
					mNoLibris = librisCheckBox.isChecked();
					mItemType = typeSpinner.getSelectedItem().toString();
					mMilieu = milieuSpinner.getSelectedItem().toString();
					
			    	//Analytic: Track Event | Category / Action / Label / Value
			        tracker.trackEvent("Skning", "Textsk: ", mQuery, 100);     
					
		            mProgressDialog = new ProgressDialog(MainSearchScreen.this);
		            mProgressDialog.setMessage("Laddar...");
	
		            mProgressThread = new ProgressThread(handler);
					mProgressDialog.show();
					mProgressThread.start();
				}
			}
			else
			{
        		Intent intent = new Intent();
        		intent = new Intent(MainSearchScreen.this, LuckyScreen.class);
        		startActivity(intent);
			}
		}
	};
	
	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) 
	{
		super.onCreate(savedInstanceState);
		
	    //Google Analytics
	    tracker = GoogleAnalyticsTracker.getInstance();
	    tracker.start(this.getString(R.string.analytics_key), this);
		
		setContentView(R.layout.main_search);
		
		Button submitButton = (Button) findViewById(R.id.submitButton);
		submitButton.setOnClickListener(onClickListener);
		
		ImageView luckyButton = (ImageView) findViewById(R.id.lucky);
		luckyButton.setOnClickListener(onClickListener);
		
	    Spinner spinner = (Spinner) findViewById(R.id.type_spinner);
	    ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(
	    this, R.array.type_array, android.R.layout.simple_spinner_item);
	    adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
	    spinner.setAdapter(adapter);
	    
	    spinner = (Spinner) findViewById(R.id.milieu_spinner);
	    adapter = ArrayAdapter.createFromResource(
	    this, R.array.milieu_array, android.R.layout.simple_spinner_item);
	    adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
	    spinner.setAdapter(adapter);		
	}	
		   
    /** 
     * Define the Handler that receives messages from the nested thread and 
     * update the GUI accordingly. In this case handling different 
     * search results and displaying messages to the user if the search 
     * did not give any results.
     */
    final Handler handler = new Handler() 
    {        
        public void handleMessage(Message msg) 
        {
        	if (msg.what == 1)
        	{
        		Intent intent = new Intent();
        		intent = new Intent(MainSearchScreen.this, ResultScreen.class);
        		startActivity(intent);
			}
        	else if (msg.what == -1)
        	{
        		Toast.makeText(getApplicationContext(), R.string.no_connection, 3000).show();
			}
        	else
        	{
        		Toast.makeText(getApplicationContext(), R.string.no_results, 3000).show();
        	}
        	mProgressDialog.dismiss();
        }
    };

    /** A nested thread that handles the actual search. */
    private class ProgressThread extends Thread {
        Handler mHandler;
       
        ProgressThread(Handler h) 
        {
            mHandler = h;
        }
       
        public void run()
        {
        	int gotData = SearchServie.getInstance().search(mQuery, mOnlyImages, mNoLibris, mItemType, mMilieu, null);
        	mHandler.sendEmptyMessage(gotData);
        }

    }
}
