package com.kringlamobil;

import java.util.List;

import com.google.android.apps.analytics.GoogleAnalyticsTracker;
import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;
import com.google.android.maps.OverlayItem;
import com.kringlamobil.R;
import com.kringlamobil.model.ListItem;
import com.kringlamobil.model.LocationOverlay;
import com.kringlamobil.model.MapOverlay;
import com.kringlamobil.model.SearchServie;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.drawable.Drawable;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.Toast;

public class MapScreen extends MapActivity 
{
	GoogleAnalyticsTracker tracker;
	
	private ProgressDialog mProgressDialog;
	private Thread mProgressThread;
	private LocationManager mLocationManager;
	private final LocationListener mGpsLocationListener = new customLocationListener();
	private final LocationListener mNetworkLocationListener = new customLocationListener();
	private MapView mMapView;
	private Location mCurrentLocation;
	private String mQuery;
	private String mCoordinates;
	private Boolean mOnlyImages = false;
	private String mItemType = "";
	private String mMilieu = "";
	private int mPage = 1;
		
	/** Called when the options menu is first created. */
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.map_menu, menu);
		return true;
	}
	
	/**
	 * Called when an item is selected in the options menu. 
	 * Depending on what item was selected different actions are taken.
	 * If the center item is selected the map centers over the users current location.
	 * If the options item is selected the 'MapOption' screen is launched.
	 * If the maptype item is selected a alert dialog lets the user choose if the map should be presented as a satellite or normal map.
	 * If the quit item is selected the application closes.
	 */
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		Intent intent = new Intent();
		switch (item.getItemId()) {
		case R.id.center:
			if (mCurrentLocation != null)
			{
    		GeoPoint geo = new GeoPoint((int)(mCurrentLocation.getLatitude()*1e6),(int)(mCurrentLocation.getLongitude()*1e6));
 	       	MapController mc = mMapView.getController();
 	       	mc.setCenter(geo);
 	       	int zoomlevel = mMapView.getMaxZoomLevel();
 	       	mc.setZoom(zoomlevel - 5);
 	       	}
			else
			{
				Toast.makeText(getApplicationContext(), R.string.no_gps, 10000).show();
			}
			return true;			
			
		case R.id.options:
			intent = new Intent(MapScreen.this, MapOptions.class);
			intent.putExtra("images", mOnlyImages);
			intent.putExtra("type", mItemType);
			intent.putExtra("milieu", mMilieu);
			startActivityForResult(intent, 0);
			return true;
			
		case R.id.maptype:
			final CharSequence[] mapeTypes = {"Normal", "Satellit"};

			AlertDialog.Builder mapTypeBuilder = new AlertDialog.Builder(this);
			mapTypeBuilder.setTitle("Vlj typ av kartlager");
			mapTypeBuilder.setItems(mapeTypes, new DialogInterface.OnClickListener() 
			{
			    public void onClick(DialogInterface dialog, int item) 
			    {			    	
			    	if (item == 1){ mMapView.setSatellite(true); }
			    	else { mMapView.setSatellite(false); }
			    }
			});
			AlertDialog mapTypeAlert = mapTypeBuilder.create();
			mapTypeAlert.show();
			return true;

		case R.id.quit:
			this.finish();
			return true;
		}

		return false;
	}
	
	/**
	 * Retrieves the parameters being sent from the map options screen.
	 * Saving them for later use.
	 */
	@Override
    protected void onActivityResult(int requestCode, int resultCode, Intent intent){
        super.onActivityResult(requestCode, resultCode, intent);
        Bundle extras = intent.getExtras();
        
        mOnlyImages = extras.getBoolean("images");
        mItemType = extras.getString("type");
        mMilieu = extras.getString("milieu");
    }
	
	/**
	 * Handles the on click event for the search button. 
	 * Retrieves the geo coordinates for the lower left and 
	 * the upper right corner, makes them into a coordinate string and 
	 * sends them together with the other parameters to the search service.
	 */
	private OnClickListener onClickListener = new OnClickListener() {
		
		public void onClick(View v) 
		{	
			if( v == findViewById(R.id.submitButton))
			{
				mPage = 1;
				EditText searchText = (EditText) findViewById(R.id.search_text);
				    
				mQuery = searchText.getText().toString();
				
		    	//Analytic: Track Event | Category / Action / Label / Value
		        tracker.trackEvent("Skning", "Kartsk: ", mQuery, 200);
				    
				InputMethodManager mgr = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				mgr.hideSoftInputFromWindow(searchText.getWindowToken(), 0);

				GeoPoint leftPoint = mMapView.getProjection().fromPixels(0, mMapView.getHeight()); 
				GeoPoint rightPoint = mMapView.getProjection().fromPixels(mMapView.getWidth(), 0); 
				    	
				mCoordinates = "%22"+(leftPoint.getLongitudeE6()/1e6)+"+"+(leftPoint.getLatitudeE6()/1e6)+"+"+(rightPoint.getLongitudeE6()/1e6)+"+"+(rightPoint.getLatitudeE6()/1e6)+"%22";
				mProgressDialog = new ProgressDialog(MapScreen.this);
				mProgressDialog.setMessage("Laddar...");
				mProgressThread = new ProgressThread(handler);
				mProgressDialog.show();
				mProgressThread.start();
			}
			else if( v == findViewById(R.id.next))
			{
				mPage ++;
				mProgressDialog = new ProgressDialog(MapScreen.this);
				mProgressDialog.setMessage("Laddar...");
				mProgressThread = new PageThread(handler);
				mProgressDialog.show();
				mProgressThread.start();
			}
			else if( v == findViewById(R.id.previous))
			{
				mPage --;
				mProgressDialog = new ProgressDialog(MapScreen.this);
				mProgressDialog.setMessage("Laddar...");
				mProgressThread = new PageThread(handler);
				mProgressDialog.show();
				mProgressThread.start();
			}
		}	
	};
	
	/** Updates the location marker. */
	private void updateWithNewLocation(Location location) 
	{
		if (location != null) 
		{
			mCurrentLocation = location;
			double lati = location.getLatitude();
			double longi = location.getLongitude();

			// xxxxxxxxxxxxxxxxx
			Drawable drawable;
			if (!mCurrentLocation.getProvider().equals("gps"))
			{
				drawable = this.getResources().getDrawable(R.drawable.red_ic_maps_indicator_current_position);
			}
			else
			{
				drawable = this.getResources().getDrawable(R.drawable.ic_maps_indicator_current_position);
			}
		    LocationOverlay locationOverlay = new LocationOverlay(drawable);
		    
		    GeoPoint point = new GeoPoint((int)(lati*1e6),(int)(longi*1e6));
		    OverlayItem overlayitem = new OverlayItem(point, "", "");
		    locationOverlay.addOverlay(overlayitem);
		    
		    updateLayer(0, locationOverlay);
		}
	}
	
	@Override
	protected void onStop() 
	{
		mLocationManager.removeUpdates(mGpsLocationListener);
		mLocationManager.removeUpdates(mNetworkLocationListener);
		super.onStop();
	}
		
	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
	    super.onCreate(savedInstanceState);
	    
	    //Google Analytics
	    tracker = GoogleAnalyticsTracker.getInstance();
	    tracker.start(this.getString(R.string.analytics_key), this);
	    tracker.trackPageView("/" +this.getClass().getName());
	    
	    setContentView(R.layout.map);
	    
	    mMapView = (MapView) findViewById(R.id.mapview);	    
	    mMapView.setBuiltInZoomControls(true);
	    mMapView.setSatellite(false);
	    	    
        MapController mc = mMapView.getController();
        mc.setCenter(new GeoPoint((int)(63.85*1e6), (int)(15.583333*1e6))); //Center on Strmsund, Jmtland, Sweden
        mc.setZoom(5);
        
	    mMapView.getOverlays().add(new Overlay() {});
	    mMapView.getOverlays().add(new Overlay() {});
	    	    
		Button submitButton = (Button) findViewById(R.id.submitButton);
		submitButton.setOnClickListener(onClickListener);
		
		ImageView next = (ImageView) findViewById(R.id.next);
		ImageView previous = (ImageView) findViewById(R.id.previous);
		next.setOnClickListener(onClickListener);
		previous.setOnClickListener(onClickListener);
			    
	    mLocationManager = (LocationManager)getSystemService(Context.LOCATION_SERVICE);
	    
	    mLocationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0, 0, mGpsLocationListener);
	    mLocationManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, 0, 0, mNetworkLocationListener);
	}

	@Override
	protected boolean isRouteDisplayed() { return false; }
	
	
	/**
	 * Makes an map overlay from the search results and then sends 
	 * them to update the map. 
	 */
	public void loadData()
	{   
	    Drawable drawable = this.getResources().getDrawable(R.drawable.marker);
	    MapOverlay itemizedoverlay = new MapOverlay(drawable, this);
		
	    List<ListItem> items = SearchServie.getInstance().getItems();
	    	    
		for (int i = 0; i < items.size(); i++) 
		{
			if (items.get(i).getCoordinates() != null)
			{
				OverlayItem overlayitem = new OverlayItem(items.get(i).getCoordinates(), null, null);
				itemizedoverlay.addOverlay(overlayitem);
			}
		}
		
		if (SearchServie.getInstance().getTotalPages() > 1)
		{
			Toast.makeText(getApplicationContext(), "Sida "+mPage +" av " +SearchServie.getInstance().getTotalPages(), 10000).show();
			
			if(SearchServie.getInstance().getTotalPages() > mPage)
			{
				findViewById(R.id.next).setVisibility(0);
			}
			else
			{
				findViewById(R.id.next).setVisibility(8);
			}
			if(mPage != 1)
			{
				findViewById(R.id.previous).setVisibility(0);
			}
			else
			{
				findViewById(R.id.previous).setVisibility(8);
			}
		}
		else
		{
			findViewById(R.id.next).setVisibility(8);
			findViewById(R.id.previous).setVisibility(8);
		}
		
	    updateLayer(1, itemizedoverlay);
	}
	
	/** 
	 * Updates a specific map layer with a new overlay.
	 * @param layer, the layer to be updated
	 * @param overLay, the new overlay
	 */
	private void updateLayer(int layer, Overlay overLay)
	{   
		List<Overlay> mapOverlays = mMapView.getOverlays();	    
	    mapOverlays.set(layer, overLay);	   
	    mMapView.invalidate();
	}
	
    /** 
     * Define the Handler that receives messages from the nested thread and 
     * update the GUI accordingly. In this case handling different 
     * search results and displaying messages to the user if the search 
     * did not give any results.
     */
    final Handler handler = new Handler() 
    {        
        public void handleMessage(Message msg) 
        {        	
        	mProgressDialog.dismiss();
        	if (msg.what == 1)
        	{
        		loadData();
			}
        	else if (msg.what == -1)
        	{
        		updateLayer(1, new Overlay(){});
        		Toast.makeText(getApplicationContext(), R.string.no_connection, 3000).show();
			}
        	else
        	{
        		updateLayer(1, new Overlay(){});
        		Toast.makeText(getApplicationContext(), R.string.no_results, 3000).show();
        	}
        }
    };

    /** A nested thread that handles the actual search. */
    private class ProgressThread extends Thread {
        Handler mHandler;
       
        ProgressThread(Handler h) 
        {
            mHandler = h;
        }
       
        public void run()
        {
        	mQuery.trim();
        	int gotData = SearchServie.getInstance().search(mQuery, mOnlyImages, false, mItemType, mMilieu, mCoordinates);
        	mHandler.sendEmptyMessage(gotData);       	
        }
    }
    
    /** A nested thread that handles the actual search. */
    private class PageThread extends Thread {
        Handler mHandler;
       
        PageThread(Handler h) 
        {
            mHandler = h;
        }
       
        public void run()
        {
        	SearchServie.getInstance().additionalSearch(mPage);
        	mHandler.sendEmptyMessage(1);       	
        }
    }
        
    /** The location listener that handles all the changes in location */
    private class customLocationListener implements LocationListener 
    {
        @Override
        public void onLocationChanged(Location location) 
        {
        	updateWithNewLocation(location);
        }
        
	    public void onProviderDisabled(String provider)
	    {
	    	updateWithNewLocation(null);
	    }

	    public void onProviderEnabled(String provider) {}

	    public void onStatusChanged(String provider, int status, Bundle extras) {}	
    }
    
}